#include <stdlib.h>
#include <stdio.h>

#include "qwindefs.h"
#include "qpushbt.h"
#include "qlined.h"
#include "qlabel.h"
#include "qbttngrp.h"
#include "qradiobt.h"
#include "qgrpbox.h"
#include "valid.h"
#include "qcombo.h"
#include "qmsgbox.h"
#include "qfile.h"

#include "gracz.h"

Gracz::Gracz()
{
    imie = NULL;
    ustawImie( DOMYSLNE_IMIE );

    oplacalnosc = 0;
    min_slowa = 2;
    max_slowa = 15;

    typ = GRACZ_WIRTUALNY;

    inicjuj();
}

Gracz::Gracz( const char *i , int o , int min , int max )
{
    imie = NULL;
    ustawImie( i );

    oplacalnosc = o;
    max_slowa = max;
    min_slowa = min;

    typ = GRACZ_WIRTUALNY;

    inicjuj();
}

Gracz::Gracz( const char *i )
{
    imie = NULL;
    ustawImie( i );

    oplacalnosc = 0;
    min_slowa = 2;
    max_slowa = 15;

    typ = GRACZ_Z_KRWI_I_KOSCI;

    inicjuj();
}

Gracz::Gracz( const Gracz &g )
{
    ustawImie( g.imie );

    for( int i=0; i<ILE_PARTII_NA_SKALPY; i++ )
	skalpy[i] = g.skalpy[i];

    oplacalnosc = g.oplacalnosc;
    max_slowa = g.max_slowa;
    min_slowa = g.min_slowa;

    ile_skalpow  = g.ile_skalpow;
    ktory_skalp  = g.ktory_skalp;
    suma_skalpow = g.suma_skalpow;

    typ = g.typ;
}

Gracz& Gracz::operator=( const Gracz &g )
{
    ustawImie( g.imie );

    for( int i=0; i<ILE_PARTII_NA_SKALPY; i++ )
	skalpy[i] = g.skalpy[i];

    oplacalnosc = g.oplacalnosc;
    max_slowa = g.max_slowa;
    min_slowa = g.min_slowa;

    ile_skalpow  = g.ile_skalpow;
    ktory_skalp  = g.ktory_skalp;
    suma_skalpow = g.suma_skalpow;

    typ = g.typ;

    return *this;
}

Gracz::~Gracz()
{
    delete imie;
}

void Gracz::inicjuj()
{
    for( int i=0; i<ILE_PARTII_NA_SKALPY; i++ )
	skalpy[i] = 0;

    ile_skalpow = 0;
    suma_skalpow = 0;
    ktory_skalp = 0;
}

void Gracz::ustawImie( const char *i )
{
    if( imie != NULL )
	delete imie;

    imie = new char[ strlen( i ) + 1 ];	// Jedynka na znak konca ciagu

    strcpy( imie , i );
}

void Gracz::dodajSkalp( int s )
{
    if( ile_skalpow < ILE_PARTII_NA_SKALPY )
    {
	skalpy[ ile_skalpow++ ] = s;
	suma_skalpow += s;
	ktory_skalp = ile_skalpow;
    }
    else
    {
	ktory_skalp == ILE_PARTII_NA_SKALPY ? ktory_skalp = 0 : ktory_skalp++;

	suma_skalpow -= skalpy[ktory_skalp];
	skalpy[ktory_skalp] = s;
	suma_skalpow += s;
    }
}

double Gracz::ranking()
{
    if( ile_skalpow < 12 )
	return 100.0;

    return ( suma_skalpow / ( ile_skalpow * 1.0 ) );
}

QTextStream & operator<<( QTextStream &wy , Gracz &g )
{
    wy << g.imie << endl;
    wy << g.ile_skalpow << " ";
    wy << g.ktory_skalp << " ";
    wy << g.oplacalnosc << " ";
    wy << g.min_slowa << " ";
    wy << g.max_slowa << " ";
    wy << g.typ << " ";

    if( g.ile_skalpow > 0 )
	wy << g.skalpy[0];

    for( int i=1; i<g.ile_skalpow; i++ )
	wy << " " << g.skalpy[i];

    return wy;
}

QTextStream & operator>>( QTextStream &we , Gracz &g )
{
    QString im;

    im = we.readLine(); we.skipWhiteSpace();
    g.ustawImie( im.ascii() );

    we >> g.ile_skalpow; we.skipWhiteSpace();
    we >> g.ktory_skalp; we.skipWhiteSpace();

    we >> g.oplacalnosc; we.skipWhiteSpace();
    we >> g.min_slowa; we.skipWhiteSpace();
    we >> g.max_slowa; we.skipWhiteSpace();
    we >> g.typ; we.skipWhiteSpace();

    g.suma_skalpow = 0;

    for( int i=0; i<g.ile_skalpow; i++ )
    {
	we >> g.skalpy[i]; we.skipWhiteSpace();
	g.suma_skalpow += g.skalpy[i];
    }

    /*
     * Pozostale wypelniamy zerami
     */
    for( int i=g.ile_skalpow; i<ILE_PARTII_NA_SKALPY; i++ )
	g.skalpy[i] = 0;

    return we;
}

/*
 * Funkcja kontrolna, napisana, bo nie chce dzialac cout
 */
void Gracz::piszOGraczu()
{
    printf( "Imi: %s\n" , imie );
    printf( "Typ : %d " , typ );
    printf( "Oplacalnosc: %d " , oplacalnosc );
    printf( "MinSlowa: %d " , min_slowa );
    printf( "MaxSlowa: %d\n" , max_slowa );
    printf( "IleSkalpow: %d " , ile_skalpow );
    printf( "SumaSkalpow: %d " , suma_skalpow );
    printf( "KtorySkalp: %d " , ktory_skalp );
    printf( "Ranking: %f\n" , ranking() );

    for( int i=0; i<ile_skalpow; i++ )
	printf( "%d " , skalpy[i] );

    printf( "\n" );
}



















/*
 * Klasa Gracze
 */

Gracze::Gracze( QWidget *parent , const char *name ) : QSemiModal( parent , name , true , WStyle_DialogBorder )
{
    QFontMetrics fm = fontMetrics();
    QString s;
    int max_size;

    setMaximumSize( 300 , 200 );
    setMinimumSize( 300 , 200 );

    dodajButton = new QPushButton( "Ok", this, "dodaj" );
    dodajButton->setGeometry(  width() / 2 - 65 , height() - 25 , 60 , 20 );

    anulujButton = new QPushButton( "Anuluj", this, "anuluj" );
    anulujButton->setGeometry( width() / 2 + 5 , height() - 25 , 60 , 20 );

    etykiety[ ET_IMIE ] = new QLabel( "Imi:" , this , "et imie" );
    etykiety[ ET_IMIE ]->move( 10 , 20 );

    imieGracza = new QLineEdit( this , "imiegracza" );
    imieGracza->setMaxLength( 23 );
    imieGracza->move( etykiety[ ET_IMIE ]->x() + 5 + fm.width( etykiety[ ET_IMIE ]->text() ) , etykiety[ ET_IMIE ]->y() );

    otoczka = new QGroupBox( "Parametry gracza wirtulanego" , this , "otoczka" );
    otoczka->setGeometry( 10 , 55 , width() - 20 , height() - 55 - 10 - 25 );

    etykiety[ ET_OPLACALNOSC ] = new QLabel( "Opacalno blanka:" , this , "et oplacalnosc" );
    etykiety[ ET_MIN_SLOWA ] = new QLabel( "Minimalna dugo sw:" , this , "et minslowa" );
    etykiety[ ET_MAX_SLOWA ] = new QLabel( "Maksymalna dugo sw:" , this , "et minslowa" );

    max_size = QMAX( fm.width( etykiety[ ET_OPLACALNOSC ]->text() ) , fm.width( etykiety[ ET_MIN_SLOWA ]->text() ) );
    max_size = QMAX( max_size , fm.width( etykiety[ ET_MAX_SLOWA ]->text() ) );

    etykiety[ ET_OPLACALNOSC ]->setGeometry( 20 , 70 , max_size , 30 );
    etykiety[ ET_MIN_SLOWA ]->setGeometry( 20 , 100 , max_size , 30 );
    etykiety[ ET_MAX_SLOWA ]->setGeometry( 20 , 130 , max_size , 30 );

    oplacalnoscBlanka = new QLineEdit( this , "oplacalnoscblanka" );
    oplacalnoscBlanka->move( etykiety[ ET_OPLACALNOSC ]->x() + 5 + max_size , etykiety[ ET_OPLACALNOSC ]->y() );

    minSlowa = new QComboBox( this , "minslowa" );
    minSlowa->move( etykiety[ ET_MIN_SLOWA ]->x() + 5 + max_size , etykiety[ ET_MIN_SLOWA ]->y() );

    maxSlowa = new QComboBox( this , "maxslowa" );
    maxSlowa->move( etykiety[ ET_MAX_SLOWA ]->x() + 5 + max_size , etykiety[ ET_MAX_SLOWA ]->y() );

    for( int i=2; i<=15; i++ )
    {
	s.setNum( i );

	minSlowa->insertItem( s );
	maxSlowa->insertItem( s );
    }

    validatory[ VALID_OPLACALNOSC ] = new MojIntValidator( 0 , 32000 , this , "val oplcalnosc" );

    oplacalnoscBlanka->setValidator( validatory[ VALID_OPLACALNOSC ] );

    typGracza = new QButtonGroup( this );
    typGracza->move( 150 , 5 );
    typGracza->resize( 140 , 50 );
    typGracza->setTitle( "Typ gracza" );
    max_size = QMAX( 80 , fm.width( "Gracz z krwi i koci" ) );
    max_size = QMAX( max_size , fm.width( "Gracz wirtualny" ) );
    max_size += 20;
    QRadioButton *rb = new QRadioButton( "Gracz wirtulany" , typGracza );
    rb->setGeometry( 5 , 15 , max_size , 15 );
    rb->setChecked( true );
    rb = new QRadioButton( "Gracz z krwi i koci" , typGracza );
    rb->setGeometry( 5 , 30 , max_size , 15 );
        
    connect( anulujButton , SIGNAL( clicked() ) , SLOT( hide() ) );
    connect( dodajButton , SIGNAL( clicked() ) , SLOT( chcemyDodacGracza() ) );
    connect( typGracza , SIGNAL( clicked( int ) ) , SLOT( enablujParametry( int ) ) );
    connect( minSlowa , SIGNAL( activated( int ) ) , SLOT( zakresMinSlowa( int ) ) );
    connect( maxSlowa , SIGNAL( activated( int ) ) , SLOT( zakresMaxSlowa( int ) ) );

    gracze = NULL;
    inicjuj( 2 );
}

Gracze::Gracze( QWidget *parent , const char *name , int i ) : QSemiModal( parent , name , true , WStyle_DialogBorder )
{
    gracze = NULL;
    inicjuj( i );
}

Gracze::~Gracze()
{
    delete [] gracze;
}

void Gracze::inicjuj( int ilu )
{
    ilu_graczy = ilu;

    if( gracze != NULL )
	delete [] gracze;

    gracze = new Gracz[ ilu_graczy ];
};

void Gracze::dodajGracza( const Gracz &g )
{
    Gracz *pom = new Gracz[ ilu_graczy + 1 ];

    for( int i=0; i<ilu_graczy; i++ )
	pom[i] = gracze[i];

    pom[ilu_graczy++] = g;

    int i = ilu_graczy - 1;

    delete [] gracze;

    gracze = pom;

    while( i > 0 )
    {
	QString s1( gracze[i].dajImie() );
	QString s2( gracze[i-1].dajImie() );
	QString ss1 = s1.lower();
	QString ss2 = s2.lower();

	if( strcmp( ss1 , ss2 ) < 0 )
	{
	    Gracz pom;

	    pom = gracze[i];
	    gracze[i] = gracze[i-1];
	    gracze[i-1] = pom;
	}

	i--;
    }

}

void Gracze::dodajGracza( const char *im , int o , int min , int max )
{
    dodajGracza( Gracz( im , o , min , max ) );
}

void Gracze::dodajGracza( const char *im )
{
    dodajGracza( Gracz( im ) );
}

Gracz *Gracze::znajdzGracza( const char *im )
{
    for( int i=0; i<ilu_graczy; i++ )
	if( strcmp( gracze[i].dajImie() , im ) == 0 )
	    return &gracze[i];

    return NULL;
}

void Gracze::zPliku()
{
    QFile plik( PLIK_Z_GRACZAMI );
    if( plik.open( IO_ReadOnly ) )
    {
	QTextStream ts( &plik );

	ts >> ilu_graczy; ts.skipWhiteSpace();

        if( gracze != NULL )
	    delete [] gracze;

	gracze = new Gracz[ ilu_graczy ];

	int ktory_gracz = 0;

	for( int i=0; i<ilu_graczy; i++ )
	    ts >> gracze[ktory_gracz++];

	plik.close();
    }
}

void Gracze::doPliku()
{
    QFile plik( PLIK_Z_GRACZAMI );

    if( plik.open( IO_WriteOnly ) )
    {
	QTextStream ts( &plik );

	ts << ilu_graczy;

	for( int i=0; i<ilu_graczy; i++ )
	    ts << endl << gracze[i];

	plik.close();
    }
}

/*
 * Tu mamy juz na glowie rysowanie
 */
void Gracze::dodajGracza()
{
    setCaption( "Dodaj gracza" );

    pokazWidgety( DO_DODANIA_GRACZA );

    show();
}

void Gracze::schowajWidgety()
{
    dodajButton->hide();
    anulujButton->hide();
    imieGracza->hide();
    typGracza->hide();

    for( int i=0; i<ILE_ETYKIET; i++ )
	etykiety[i]->hide();

    oplacalnoscBlanka->hide();
    maxSlowa->hide();
    minSlowa->hide();
    otoczka->hide();
}

void Gracze::pokazWidgety( int do_czego )
{
    schowajWidgety();

    switch( do_czego )
    {
	case DO_DODANIA_GRACZA:
	    dodajButton->show();
	    anulujButton->show();
	    imieGracza->show();
	    typGracza->show();
	    otoczka->show();
	    etykiety[ ET_IMIE ]->show();
	    etykiety[ ET_OPLACALNOSC ]->show();
	    etykiety[ ET_MIN_SLOWA ]->show();
	    etykiety[ ET_MAX_SLOWA ]->show();
	    oplacalnoscBlanka->show();
	    minSlowa->show();
	    maxSlowa->show();
	    
	    imieGracza->setText( tr( DOMYSLNE_IMIE ) );
	    oplacalnoscBlanka->setText( "0" );
	    minSlowa->setCurrentItem( 0 );
	    maxSlowa->setCurrentItem( 13 );

	    break;

	default:;
    }
}

void Gracze::enablujParametry( int co_wcisnieto )
{
    int na_co = true;

    switch( co_wcisnieto )
    {
	case RADIO_WIRTUALNY:
	    na_co = true;
	    break;

	case RADIO_Z_KRWI:
	    na_co = false;
	    break;

	default:;
    }

    oplacalnoscBlanka->setEnabled( na_co );
    minSlowa->setEnabled( na_co );
    maxSlowa->setEnabled( na_co );
}

void Gracze::zakresMinSlowa( int index )
{
    QString s1 = minSlowa->text( index ), s2 = maxSlowa->currentText();

    if( s1.toInt() > s2.toInt() )
    {
	minSlowa->setCurrentItem( s2.toInt() - 2 );
	return;
    }
}

void Gracze::zakresMaxSlowa( int index )
{
    QString s1 = minSlowa->currentText(), s2 = maxSlowa->text( index );

    if( s1.toInt() > s2.toInt() )
    {
	maxSlowa->setCurrentItem( s1.toInt() - 2 );
	return;
    }
}

void Gracze::chcemyDodacGracza()
{
    QString s = imieGracza->text();

    if( strlen( imieGracza->text() ) == 0 )
    {
	QMessageBox::information( this , "Ze imi" , "Graczowi naley wybra jakie imi." );
	return;
    }

    if( znajdzGracza( imieGracza->text() ) != NULL )
    {
	QMessageBox::information( this , "Ze imi" , "Jest ju gracz o takim imieniu." );
	return;
    }

    if( oplacalnoscBlanka->isEnabled() )
    {
	QString s1 = oplacalnoscBlanka->text();

	dodajGracza( s , s1.toInt() , minSlowa->currentItem() + 2 , maxSlowa->currentItem() + 2 );
    }
    else
	dodajGracza( s );

    hide();

    emit graczDodany();
}

Gracz& Gracze::operator[]( int elem )
{
    if( elem >= 0 && elem < ilu_graczy )
	return gracze[elem];

    exit(0);
}

void Gracze::ustawKtorzyGraja( int ktory , const QString &im )
{
    for( int i=0; i<ilu_graczy; i++ )
	if( QString( gracze[i].dajImie() ) == im.right( im.length() - 2 ) ) // -2, bo w comboboksach 2 znaczki na poczatku
	{
	    ktorzy_graja[ktory] = i;
	    return;
	}
}
