#ifndef GRACZ_H
#define GRACZ_H

#include <qtstream.h>
#include <qsemimodal.h>

#include "dupa.h"

#define ILE_PARTII_NA_SKALPY 200
#define PLIK_Z_GRACZAMI "./dane_graczy"
#define DOMYSLNE_IMIE "todzib"

class Gracz
{
	friend QTextStream& operator<<( QTextStream& , Gracz& );
	friend QTextStream& operator>>( QTextStream& , Gracz& );

    public:
	Gracz();
	Gracz( const char * );
	Gracz( const char * , int , int , int );
	Gracz( const Gracz & );
	~Gracz();

	Gracz& operator=( const Gracz & );

	void ustawImie( const char * );
	char *dajImie() { return imie; }
	int dajMinSlowa() { return min_slowa; }
	int dajMaxSlowa() { return max_slowa; }
	int dajOplacalnoscBlanka() { return oplacalnosc; }
	void dodajSkalp( int );
	double ranking();
	void piszOGraczu();
	int graczWirtualny() { return typ == GRACZ_WIRTUALNY; }
	int graczZKrwi() { return typ == GRACZ_Z_KRWI_I_KOSCI; }

    private:
	void inicjuj();

	char *imie;
	int skalpy[ ILE_PARTII_NA_SKALPY ];
	int ile_skalpow, suma_skalpow, ktory_skalp;
	int oplacalnosc, min_slowa, max_slowa;
	int typ;
};

#define DO_DODANIA_GRACZA 0

#define ILE_ETYKIET 4
#define ET_IMIE 0
#define ET_OPLACALNOSC 1
#define ET_MIN_SLOWA 2
#define ET_MAX_SLOWA 3

#define RADIO_WIRTUALNY 0
#define RADIO_Z_KRWI 1

#define ILE_VALIDATOROW 1
#define VALID_OPLACALNOSC 0

class QPushButton;
class QLineEdit;
class QLabel;
class QButtonGroup;
class QGroupBox;
class QComboBox;
class MojIntValidator;

class Gracze : public QSemiModal
{
    Q_OBJECT

    public:
	Gracze( QWidget * , const char * );
	Gracze( QWidget * , const char * , int );
	~Gracze();

	Gracz& operator[]( int );

	void dodajGracza( const char * );
	void dodajGracza( const char * , int , int , int );
	void dodajGracza( const Gracz & );
	Gracz *znajdzGracza( const char * );
	void doPliku();
	void zPliku();
	int iluGraczy() { return ilu_graczy; }
	Gracz *dajGracza( int ktorego ) { return &gracze[ktorego]; }
	Gracz *dajGrajacego( int ktorego ) { return &gracze[ ktorzy_graja[ktorego] ]; }
	void ustawKtorzyGraja( int ktory , int kto ) { ktorzy_graja[ktory] = kto; }
	void ustawKtorzyGraja( int , const QString & );

	void pokazWidgety( int );
	void schowajWidgety();

    public slots:
	void dodajGracza();
	void enablujParametry( int );
	void zakresMinSlowa( int );
	void zakresMaxSlowa( int );
	void chcemyDodacGracza();

    signals:
	void graczDodany();

    private:
	void inicjuj( int );

	QPushButton *dodajButton, *anulujButton;
	QLineEdit *imieGracza, *oplacalnoscBlanka;
	QComboBox *minSlowa, *maxSlowa;
	QLabel *etykiety[ ILE_ETYKIET ];
	MojIntValidator *validatory[ ILE_VALIDATOROW ];
	QButtonGroup *typGracza;
	QGroupBox *otoczka;

	Gracz *gracze;
	int ilu_graczy;
	int ktorzy_graja[2];	// Ktorzy gracze wlasnie tocza boj
};
#endif
